window.Gleam.initSearch([{"doc":"cors_builder","title":"cors_builder","content":"# CORS Builder\n\n&gt; [!IMPORTANT]\n&gt;\n&gt; Before diving in CORS,\n&gt; [make sure you&#39;re aware of security advices](#more-details--notes-about-security)\n&gt; and see if you can&#39;t just use a simple proxy to avoid CORS! It&#39;s a better and\n&gt; more secure way to manage CORS! Always secure correctly your CORS, and use\n&gt; them sparingly, when needed.\n\nManipulating CORS is often a pain for developers, and always a little blurry, to\nunderstand what should be done, how it should be configured, etc. CORS Builder\nabstract the complexity while trying to remains simple, and friendly warns you\nwhen something is wrong.\n\nCORS Builder is compatible with every servers, as long as you&#39;re using the\n[`gleam_http`](https://hexdocs.pm/gleam_http) `Response` as a foundation.\nHowever, to simplify your development, two middlewares are provided\nout-of-the-box:\n[`wisp_middleware`](https://hexdocs.pm/cors_builder/cors_builder.html#wisp_middleware)\nand\n[`mist_middleware`](https://hexdocs.pm/cors_builder/cors_builder.html#mist_middleware)\nto integrate nicely in [`wisp`](https://hexdocs.pm/wisp) and\n[`mist`](https://hexdocs.pm/mist). You should never have to worry about CORS\nagain! Use the package, configure your CORS, and everything should work\nsmoothly!\n\n## Quickstart\n\nYou can interchange `wisp_middleware` with `mist_middleware` if you&#39;re using\n`wisp` or `mist`.\n\n```gleam\nimport cors_builder as cors\nimport gleam/http\nimport mist\nimport wisp.{type Request, type Response}\n\nfn cors() {\n  cors.new()\n  |&gt; cors.allow_origin(&quot;http://localhost:3000&quot;)\n  |&gt; cors.allow_origin(&quot;http://localhost:4000&quot;)\n  |&gt; cors.allow_method(http.Get)\n  |&gt; cors.allow_method(http.Post)\n}\n\nfn handler(req: Request) -&gt; Response {\n  use req &lt;- cors.wisp_middleware(req, cors())\n  wisp.ok()\n}\n\nfn main() {\n  handler\n  |&gt; wisp.mist_middlewarer(secret_key)\n  |&gt; mist.new()\n  |&gt; mist.port(3000)\n  |&gt; mist.start_http()\n}\n```\n\n## More details &amp; notes about security\n\nCORS are often badly understood, however they&#39;re full parts of the web stack\nwhen working with browsers, and they&#39;re part of security measures, to avoid\nusers&#39; browsers behaving badly.\n\nCORS intervene when browsers have to manage with cross-origin requests. A\ncross-origin request is a request coming from a different domain than the domain\nyou&#39;re currently on. Imagine you&#39;re browsing your favorite website, like\n[packages.gleam.run](https://packages.gleam.run), and suddenlly, your browser\nwant to query [google.com](https://google.com) in an async way. Because you&#39;re\nnot on Google, the browser will identify your request as a cross-origin request.\nSome more security measures have to be taken to make sure the request is valid.\nThat&#39;s where CORS comes into play. CORS stands for Cross-Origin Resource\nSharing. It means it&#39;s a way to authorize cross-origin requests, to allow\noutside clients to access the desired resources.\n\nThis mechanism is a way to prevent browsers to ask for data on behalf of a user,\nin an undesired way. It&#39;s up to you, when developping your server, to make sure\nonly authentified, regular users can access your service. **It is a bad idea to\nlet everyone access your data directly from a browser.** You should identify who\ncan access your service, and how, that&#39;s what CORS are made for. Most of the\ntime, you want your frontend to access your backend, and nothing else. You can\nsimply identify those domains, and add them in your CORS configuration. Let&#39;s\nimagine your frontend is hosted on `https://frontend.app` and your backend on\n`https://backend.app`. You can configure your CORS to _only_ accept\n`https://frontend.app`. That way, every request coming from another domain will\nbe rejected, and only your users will be accepted.\n\nKeep in mind that CORS will never trigger as long as your frontend query the\nsame domain where it resides. When your frontend queries\n`https://frontend.app/api/path` for example, because your frontend resides on\n`https://frontend.app`, no cross-origin request is identified, so CORS won&#39;t\ncomes into play. So always think about this, and see if you can just host your\nfrontend at the same address as your backend. This can be achieved using a\nproxy, and this should be soon available in lustre dev tools, and is already\navailable if you&#39;re using\n[Vite](https://vitejs.dev/config/server-options#server-proxy) or\n[Webpack](https://webpack.js.org/configuration/dev-server/#devserverproxy)!\n\n## How are CORS working?\n\nBrowsers apply a simple rules for every HTTP request: when the request\noriginates from a different origin than the target server URL — and if it&#39;s not\na simple request — the browser needs to authorize the cross-origin call.\n\n&gt; From the HTTP point of view, a simple request respects the following\n&gt; conditions:\n&gt;\n&gt; - Allowed methods are `GET`, `HEAD` or `POST`\n&gt; - Allowed headers are `accept`, `accept-language`, `content-language` and\n&gt;   `content-type`\n&gt; - `content-type` should be:\n&gt;   - `application/x-www-form-urlencoded`\n&gt;   - `multipart/form-data`\n&gt;   - `text/plain`\n&gt; - No event listener has been added on `XMLHttpRequestUpload`.\n&gt;   `XMLHttpRequestUpload.upload` is preferred.\n&gt; - No `ReadableStream` is used in the request.\n\nTo authorize the call, the browser will issue a first request, called a\n&quot;preflight&quot; request. This request takes the form of an `OPTIONS` request, which\nshould be answered positively by the server (meaning the response status code\nshould be 2XX) and should contains the appropriate CORS headers\n(`access-control` headers).\n\nIn case the preflight request is not successful, the server will simply cancel\nthe HTTP request. But if the preflight request is successful, then the browser\nwill then launch the real request, and the server will be able to handle it.\n\n## What are the headers?\n\nWe distinguish different types of headers: the headers concerning the request\nissuer (the caller) and the headers responded by the server.\n\n&gt; [!NOTE]\n&gt;\n&gt; In HTTP, all headers keys are case-insensitive. It means all headers can be\n&gt; written as `content-type` or `Content-Type` or even `CONTENT-type`. By\n&gt; convention, they&#39;re written as `Content-Type`. In HTTP2 though, all headers\n&gt; keys have to be lowercase or the requests are rejected, and `gleam_http` will\n&gt; enforce this behaviour. All headers keys in this guide will be written in\n&gt; lowercase. On the internet you could still see both way of writing them.\n\n### Response headers\n\nResponse headers are not automatically set by the server, and you should handle\nthem according on what you want to do. This package tries to abstract it to\nsimplify your development and let you focus on your application. We count 6 CORS\nresponse headers:\n\n- `access-control-allow-origin`, indicates which origins are allowed to access\n  the server. It can be a joker (`&quot;*&quot;`) or a unique domain\n  (`https://gleam.run`). It cannot contains multiple domains, but can response\n  to multiple different domains with the `vary` header. You should not have to\n  take care of this, because the library provides it for you.\n- `access-control-expose-headers`, provides a whitelist of allowed headers for\n  the browsers. Only the headers in the whitelist will be able to be used in the\n  response object in the JS code. It means if the response contains headers you\n  want to cache to the client, you can use this header.\n- `access-control-max-age`, allows to put the preflight response in cache, for a\n  specified amount of time. This avoids to rerun the `OPTIONS` request multiple\n  times.\n- `access-control-allow-credentials`, allows the request to includes credentials\n  authorizations. This can expose you to CSRF attack. Never activate this option\n  unless you carefully know what you&#39;re doing.\n- `access-control-allow-methods`, provides a whitelist of subsequent authorized\n  methods in the future requests.\n- `access-control-allow-headers`, indicates which headers are accepted by the\n  server, and thus, which headers the browser will be able to send in subsequent\n  requests.\n\n### Request headers\n\nRequest headers are headers automatically set by the browser, when issuing a\nrequest with `XMLHttpRequest` or `fetch`. You should not bother about it, but\nthey&#39;re still referenced it, in case you encounter them.We count 3 CORS request\nheaders:\n\n- `origin` contains the origin of the request. The browser will _always_ fill\n  this header automatically.\n- `access-control-request-method` contains the desired methods to use when\n  talking with the server.\n- `access-control-request-header` contains the desired headers that the request\n  want to have.\n\n## Contributing\n\nYou love the package and want to improve it? You have a shiny new framework and\nwant to provide an integration with CORS in this package? Every contribution is\nwelcome! Feel free to open a Pull Request, and let&#39;s discuss about it!\n","url":"index.html"},{"doc":"cors_builder","title":"Cors","content":"pub opaque type Cors\n CORS builder. Use it in your program to generate the good CORS for your\n responses.\n\nSynonyms:\ncors_builder.Cors\ncors_builder Cors","url":"cors_builder.html#Cors"},{"doc":"cors_builder","title":"Origin","content":"pub opaque type Origin\n Indicates the origin for CORS. Could be any origin (wildcard `&quot;*&quot;`) or a\n list of domains. In case it&#39;s a list of domains, one domain will be returned\n every time, and the `vary` header will be filled with `origin`.\n If you only have one domain, the domain will always be filled.\n\nSynonyms:\ncors_builder.Origin\ncors_builder Origin","url":"cors_builder.html#Origin"},{"doc":"cors_builder","title":"allow_all_origins","content":"pub fn allow_all_origins(cors: Cors) -&gt; Cors\n Allow all domains to access your server.\n\n Be extremely careful, you should not use this function in production!\n Allowing all origins can easily be a huge security flaw!\n Allow only the origins you need, and use this function only locally,\n in dev mode.\nSynonyms:\ncors_builder.allow_all_origins\ncors_builder allow_all_origins","url":"cors_builder.html#allow_all_origins"},{"doc":"cors_builder","title":"allow_credentials","content":"pub fn allow_credentials(cors: Cors) -&gt; Cors\n Allow credentials to be sent in the request. Credentials take form of\n username and password, stored in cookies most of the time.\n\n Be extremely careful with this header, and consider it with caution, mainly\n for legacy systems relying on cookies or for systems aware of the danger of\n cookies, because of [CSRF attacks](https://developer.mozilla.org/en-US/docs/Glossary/CSRF).\n You probably don&#39;t really need it if you use lustre or any modern framework\n you&#39;ll find in the gleam ecosystem!\n\n When you can, prefer using some modern system, like OAuth2 or rely on a\n framework doing the authentication for you. A simple and secured way to\n authenticate your users is to use the `authorization` header, with a `Bearer`\n token.\nSynonyms:\ncors_builder.allow_credentials\ncors_builder allow_credentials","url":"cors_builder.html#allow_credentials"},{"doc":"cors_builder","title":"allow_header","content":"pub fn allow_header(cors: Cors, header: String) -&gt; Cors\n All header to be sent to the server.\n You can specify multiple headers to send to your server. In this case, call\n the function multiple times on `Cors` data.\n \n fn cors() {\n   cors.new()\n   |&gt; cors.allow_header(&quot;content-type&quot;)\n   |&gt; cors.allow_header(&quot;origin&quot;)\n }\n \nSynonyms:\ncors_builder.allow_header\ncors_builder allow_header","url":"cors_builder.html#allow_header"},{"doc":"cors_builder","title":"allow_method","content":"pub fn allow_method(cors: Cors, method: Method) -&gt; Cors\n Allow methods to be used in subsequent CORS requests.\n You can specify multiple allowed methods. In this case, call the function\n multiple times on `Cors` data.\n \n import gleam/http\n\n fn cors() {\n   cors.new()\n   |&gt; cors.allow_method(http.Get)\n   |&gt; cors.allow_method(http.Post)\n }\n \nSynonyms:\ncors_builder.allow_method\ncors_builder allow_method","url":"cors_builder.html#allow_method"},{"doc":"cors_builder","title":"allow_origin","content":"pub fn allow_origin(cors: Cors, origin: String) -&gt; Cors\n Allow a specific domain to access your server. The domain must be a valid\n URI, conformant to RFC 3986. In case it&#39;s not conformant, a warning will be\n emitted, and Cors won&#39;t be changed.\n You can specify multiple domains to access your server. In this case, call\n the function multiple times on `Cors` data.\n \n fn cors() {\n   cors.new()\n   |&gt; cors.allow_origin(&quot;domain&quot;)\n   |&gt; cors.allow_origin(&quot;domain2&quot;)\n }\nSynonyms:\ncors_builder.allow_origin\ncors_builder allow_origin","url":"cors_builder.html#allow_origin"},{"doc":"cors_builder","title":"expose_header","content":"pub fn expose_header(cors: Cors, header: String) -&gt; Cors\n Expose headers in the resulting request.\n You can specify multiple headers to access your server. In this case, call\n the function multiple times on `Cors` data.\n \n fn cors() {\n   cors.new()\n   |&gt; cors.expose_header(&quot;content-type&quot;)\n   |&gt; cors.expose_header(&quot;vary&quot;)\n }\n \nSynonyms:\ncors_builder.expose_header\ncors_builder expose_header","url":"cors_builder.html#expose_header"},{"doc":"cors_builder","title":"max_age","content":"pub fn max_age(cors: Cors, age: Int) -&gt; Cors\n Set an amount of seconds during which CORS requests can be cached.\n When using `max_age`, the browser will issue one request `OPTIONS` at first,\n and will reuse the result of that request for the specified amount of time.\n Once the cache expired, a new `OPTIONS` request will be made.\nSynonyms:\ncors_builder.max_age\ncors_builder max_age","url":"cors_builder.html#max_age"},{"doc":"cors_builder","title":"mist_middleware","content":"pub fn mist_middleware(\n  req: Request(Connection),\n  cors: Cors,\n  handler: fn(Request(Connection)) -&gt; Response(ResponseData),\n) -&gt; Response(ResponseData)\n Intercepts the request for mist and handles CORS directly without worrying\n about it. Provide your CORS configuration, and you&#39;re good to go!\nSynonyms:\ncors_builder.mist_middleware\ncors_builder mist_middleware","url":"cors_builder.html#mist_middleware"},{"doc":"cors_builder","title":"new","content":"pub fn new() -&gt; Cors\n Creates an empty CORS object. It will not contains anything by default.\n If you&#39;re using it directly, no headers will be added to the response.\nSynonyms:\ncors_builder.new\ncors_builder new","url":"cors_builder.html#new"},{"doc":"cors_builder","title":"set_cors","content":"pub fn set_cors(res: Response(a), cors: Cors) -&gt; Response(a)\n Set CORS headers on a response. Should be used in your handler.\n In case you&#39;re using a framework, it probably already implements it.\n If you&#39;re using mist or wisp, use the corresponding provided middlewares,\n ([mist_middleware](#mist_middleware)) and ([wisp_middleware](#wisp_middleware)) and do not\n use this &quot;low-level&quot; function.\nSynonyms:\ncors_builder.set_cors\ncors_builder set_cors","url":"cors_builder.html#set_cors"},{"doc":"cors_builder","title":"set_cors_multiple_origin","content":"pub fn set_cors_multiple_origin(\n  res: Response(a),\n  cors: Cors,\n  origin: String,\n) -&gt; Response(a)\n Set CORS headers on a response. Should be used when you have multiple\n allowed domains. Should be used in your handler.\n In case you&#39;re using a framework, it probably already implements it.\n If you&#39;re using mist or wisp, use the corresponding provided middlewares,\n ([mist_middleware](#mist_middleware)) and ([wisp_middleware](#wisp_middleware)) and do not\n use this &quot;low-level&quot; function.\nSynonyms:\ncors_builder.set_cors_multiple_origin\ncors_builder set_cors_multiple_origin","url":"cors_builder.html#set_cors_multiple_origin"},{"doc":"cors_builder","title":"wisp_middleware","content":"pub fn wisp_middleware(\n  req: Request(Connection),\n  cors: Cors,\n  handler: fn(Request(Connection)) -&gt; Response(Body),\n) -&gt; Response(Body)\n Intercepts the request for wisp and handles CORS directly without worrying\n about it. Provide your CORS configuration and you&#39;re good to go!\nSynonyms:\ncors_builder.wisp_middleware\ncors_builder wisp_middleware","url":"cors_builder.html#wisp_middleware"},{"doc":"cors_builder","title":"cors_builder","content":" `cors_builder` provides an easy way to build and inject your CORS configuration\n in your server. The package tries to remains as simplest as possible, while\n guaranteeing type-safety and correctness of the CORS configuration.\n\n ## Quickstart\n\n Import the `cors_builder` package, and configure your CORS. Finally, use the\n correct corresponding middleware for your server, and you&#39;re done!\n\n ```\n import cors_builder as cors\n import gleam/http\n import mist\n import wisp.{type Request, type Response}\n\n // Dummy example.\n fn cors() {\n   cors.new()\n   |&gt; cors.allow_origin(&quot;http://localhost:3000&quot;)\n   |&gt; cors.allow_origin(&quot;http://localhost:4000&quot;)\n   |&gt; cors.allow_method(http.Get)\n   |&gt; cors.allow_method(http.Post)\n }\n\n fn handler(req: Request) -&gt; Response {\n   use req &lt;- cors.wisp_middleware(req, cors())\n   wisp.ok()\n }\n\n fn main() {\n   handler\n   |&gt; wisp.mist_handler(secret_key)\n   |&gt; mist.new()\n   |&gt; mist.port(3000)\n   |&gt; mist.start_http()\n }\n ```\n\n ## Low-level functions\n\n If you&#39;re building your framework or you know what you&#39;re doing, you should\n take a look at [`set_cors`](#set_cors) and\n [`set_cors_multiple_origin`](#set_cors_multiple_origin). They allow to\n inject the CORS in your response, and it allows you to create your\n middleware to use with the bare CORS data.\n\n If you&#39;re not building your framework, you should _probably_ heads to [`wisp`](https://hexdocs.pm/wisp)\n to get you started. It&#39;s better to familiarize with the ecosystem before\n jumping right in your custom code.","url":"cors_builder.html"}]);